@@ -162,8 +162,7 @@ static void init_stack (MVMThreadContext *tc, collation_stack *stack) {
 }
 static void cleanup_stack (MVMThreadContext *tc, collation_stack *stack) {
     if (stack->keys != NULL) {
-        MVM_free(stack->keys);
-        stack->keys = NULL;
+        MVM_free_null(stack->keys);
     }
 }
 static void push_key_to_stack(collation_stack *stack, MVMuint32 primary, MVMuint32 secondary, MVMuint32 tertiary) {\
@@ -236,7 +235,7 @@ static MVMint32 NFD_and_push_collation_values (MVMThreadContext *tc, MVMCodepoin
     return rtrn;
 }
 /* Returns the number of collation elements pushed onto the stack */
-static MVMint32 collation_push_MVM_values (MVMThreadContext *tc, MVMCodepoint cp, collation_stack *stack, MVMCodepointIter *ci, char *name) {
+static void collation_push_MVM_values (MVMThreadContext *tc, MVMCodepoint cp, collation_stack *stack, MVMCodepointIter *ci, char *name) {
     collation_key MVM_coll_key = {
         MVM_unicode_collation_primary(tc, cp), MVM_unicode_collation_secondary(tc, cp), MVM_unicode_collation_tertiary(tc, cp), 0
     };
@@ -244,7 +243,9 @@ static MVMint32 collation_push_MVM_values (MVMThreadContext *tc, MVMCodepoint cp
      * Eventually we might want to restructure this code here */
     if (is_Block_Tangut(cp) || MVM_coll_key.s.primary <= 0 || MVM_coll_key.s.secondary <= 0 || MVM_coll_key.s.tertiary <= 0) {
         MVMuint32 AAAA, BBBB;
+#ifdef COLLATION_DEBUG
         char *block_pushed = NULL;
+#endif
         collation_key calculated_key[2] = {
             {0, 0x20, 0x2, 0},
             {0, 0x00, 0x0, 0}
@@ -278,7 +279,7 @@ static MVMint32 collation_push_MVM_values (MVMThreadContext *tc, MVMCodepoint cp
         else {
             MVMint32 NFD_rtrn = NFD_and_push_collation_values(tc, cp, stack, ci, name);
             if (NFD_rtrn) {
-                return NFD_rtrn;
+                return;
             }
             else {
                 AAAA = compute_AAAA(cp, 0xFBC0);
@@ -290,14 +291,12 @@ static MVMint32 collation_push_MVM_values (MVMThreadContext *tc, MVMCodepoint cp
         calculated_key[1].s.primary = BBBB;
         DEBUG_PRINT_SPECIAL_PUSHED(block_pushed, name, cp);
         push_onto_stack(tc, stack, calculated_key, 2, name);
-        return 2;
     }
     else {
         push_key_to_stack(stack,
             MVM_coll_key.s.primary,
             MVM_coll_key.s.secondary,
             MVM_coll_key.s.tertiary);
-        return 1;
     }
 }
 /* This is passed the terminal node and it adds the collation elements linked from
@@ -308,7 +307,7 @@ static MVMint32 collation_push_MVM_values (MVMThreadContext *tc, MVMCodepoint cp
  * Essentially the return value lets you know if it ended up pushing collation values for the last codepoint
  * in the sequence or if it only pushed collation values for fallback_cp
 */
-MVMint64 collation_add_keys_from_node (MVMThreadContext *tc, sub_node *last_node, collation_stack *stack, MVMCodepointIter *ci, char *name, MVMCodepoint fallback_cp, sub_node *first_node) {
+static int collation_add_keys_from_node (MVMThreadContext *tc, sub_node *last_node, collation_stack *stack, MVMCodepointIter *ci, char *name, MVMCodepoint fallback_cp, sub_node *first_node) {
     MVMint64 j;
     MVMint64 rtrn = 0;
     sub_node *choosen_node = NULL;
@@ -335,7 +334,7 @@ MVMint64 collation_add_keys_from_node (MVMThreadContext *tc, sub_node *last_node
     collation_push_MVM_values(tc, fallback_cp, stack, ci, name);
     return rtrn;
 }
-MVMint64 find_next_node (MVMThreadContext *tc, sub_node node, MVMCodepoint next_cp) {
+static MVMint64 find_next_node (MVMThreadContext *tc, sub_node node, MVMCodepoint next_cp) {
     MVMint64 next_min, next_max;
     MVMint64 i;
     /* There is nowhere else to go */
@@ -352,10 +351,9 @@ MVMint64 find_next_node (MVMThreadContext *tc, sub_node node, MVMCodepoint next_
     }
     return -1;
 }
-MVMint64 get_main_node (MVMThreadContext *tc, int cp, int range_min, int range_max) {
+static MVMint64 get_main_node (MVMThreadContext *tc, int cp, int range_min, int range_max) {
     MVMint64 i;
     MVMint64 rtrn = -1;
-    int counter   = 0;
     /* Decrement range_min because binary search defaults to 1..* not 0..* */
     range_min--;
     /* starter_main_nodes_elems are all the nodes which are the origin nodes
@@ -375,11 +373,9 @@ MVMint64 get_main_node (MVMThreadContext *tc, int cp, int range_min, int range_m
 }
 /* Returns the number of added collation keys */
 static MVMint64 collation_push_cp (MVMThreadContext *tc, collation_stack *stack, MVMCodepointIter *ci, int *cp_maybe, int cp_num, char *name) {
-    MVMint64 rtrn = 0;
     MVMCodepoint cps[10];
     MVMint64 num_cps_processed = 0;
     int query = -1;
-    int cp_num_orig = cp_num;
     /* If supplied -1 that means we need to grab it from the codepoint iterator. Otherwise
      * the value we were passed is the codepoint we should process */
     if (cp_num == 0) {
@@ -392,7 +388,14 @@ static MVMint64 collation_push_cp (MVMThreadContext *tc, collation_stack *stack,
             cps[i] = cp_maybe[i];
         }
     }
+#if defined(__GNUC__) && !defined(__clang__)
+    #pragma GCC diagnostic push
+    #pragma GCC diagnostic ignored "-Wmaybe-uninitialized"
+#endif
     query = get_main_node(tc, cps[0], 0, starter_main_nodes_elems);
+#if defined(__GNUC__) && !defined(__clang__)
+    #pragma GCC diagnostic pop
+#endif
     if (query != -1) {
         DEBUG_PRINT_SUB_NODE(main_nodes[query]);
         /* If there are no sub_node_elems that means we don't need to look at
@@ -422,8 +425,9 @@ static MVMint64 collation_push_cp (MVMThreadContext *tc, collation_stack *stack,
                     last_good_i      = i;
                     last_good_result = result;
                 }
-                if (result != -1)
+                if (result != -1) {
                     DEBUG_PRINT_SUB_NODE(main_nodes[result]);
+                }
             }
             /* If there is no last_good_result we should return a value from main_nodes */
             DEBUG_PRINT_SUB_NODE( (last_good_result == -1 ? main_nodes[query] : main_nodes[last_good_result]) );
@@ -439,7 +443,7 @@ static MVMint64 collation_push_cp (MVMThreadContext *tc, collation_stack *stack,
     }
     else {
         /* Push the first codepoint onto the stack */
-        rtrn = collation_push_MVM_values(tc, cps[0], stack, ci, name);
+        collation_push_MVM_values(tc, cps[0], stack, ci, name);
         num_cps_processed = 1;
     }
     /* If there are any more codepoints remaining call collation_push_cp on the remaining */
@@ -448,14 +452,13 @@ static MVMint64 collation_push_cp (MVMThreadContext *tc, collation_stack *stack,
     }
     return num_cps_processed;
 }
-MVMint64 grab_from_stack(MVMThreadContext *tc, MVMCodepointIter *ci, collation_stack *stack, char *name) {
+static int grab_from_stack(MVMThreadContext *tc, MVMCodepointIter *ci, collation_stack *stack, char *name) {
     if (!MVM_string_ci_has_more(tc, ci))
         return 0;
     collation_push_cp(tc, stack, ci, NULL, 0, name);
     return 1;
 }
 static void init_ringbuffer (MVMThreadContext *tc,  ring_buffer *buffer) {
-    MVMint64 i;
     buffer->count           =  0;
     buffer->location        = -1;
     buffer->codes_out_count =  0;
@@ -507,7 +510,6 @@ MVMint64 MVM_unicode_string_compare(MVMThreadContext *tc, MVMString *a, MVMStrin
     MVMStringIndex alen, blen;
     /* Iteration variables */
     MVMCodepointIter a_ci, b_ci;
-    MVMGrapheme32 ai, bi;
     /* Set it all to 0 to start with. We alter this based on the collation_mode later on */
     level_eval level_eval_settings = {
         { {0,0,0}, {0,0,0}, {0,0,0}, {0,0,0} }
@@ -524,11 +526,10 @@ MVMint64 MVM_unicode_string_compare(MVMThreadContext *tc, MVMString *a, MVMStrin
     /* This value stores what the return value would be if the strings were compared
      * by codepoint. This is used to break collation value ties */
     MVMint64 compare_by_cp_rtrn = 0;
-    MVMint64 pos_a = 0, pos_b = 0, i = 0, rtrn = 0;
+    MVMint64 pos_a = 0, pos_b = 0, rtrn = 0;
     MVMint16 grab_a_done = 0, grab_b_done = 0;
     /* From 0 to 2 for primary, secondary, tertiary levels */
     MVMint16   level_a = 0,   level_b = 0;
-    MVMint64 skipped_a = 0, skipped_b = 0;
     /* This code sets up level_eval_settings based on the collation_mode */
     #define setmodeup(mode, level, Less, Same, More) {\
         if (collation_mode & mode) {\
@@ -603,12 +604,10 @@ MVMint64 MVM_unicode_string_compare(MVMThreadContext *tc, MVMString *a, MVMStrin
              * of a level separator so it is evaluated as the end of the string, causing the shorter string to win */
             if (stack_a.keys[pos_a].a[level_a] == collation_zero) {
                 pos_a++;
-                skipped_a++;
                 continue;
             }
             if (stack_b.keys[pos_b].a[level_b] == collation_zero) {
                 pos_b++;
-                skipped_b++;
                 continue;
             }
             /* If collation values are not equal */
@@ -678,14 +677,11 @@ MVMint64 MVM_unicode_string_compare(MVMThreadContext *tc, MVMString *a, MVMStrin
 
 /* Looks up a codepoint by name. Lazily constructs a hash. */
 MVMGrapheme32 MVM_unicode_lookup_by_name(MVMThreadContext *tc, MVMString *name) {
-    MVMuint64 size;
     char *cname = MVM_string_utf8_encode_C_string(tc, name);
-    size_t cname_len = strlen((const char *) cname );
-    MVMUnicodeNameRegistry *result;
-    if (!codepoints_by_name) {
+    if (MVM_uni_hash_is_empty(tc, &tc->instance->codepoints_by_name)) {
         generate_codepoints_by_name(tc);
     }
-    HASH_FIND(hash_handle, codepoints_by_name, cname, cname_len, result);
+    struct MVMUniHashEntry *result = MVM_uni_hash_fetch(tc, &tc->instance->codepoints_by_name, cname);
     if (!result) {
         #define prefixes_len 7
         const char *prefixes[prefixes_len] = {
@@ -697,9 +693,10 @@ MVMGrapheme32 MVM_unicode_lookup_by_name(MVMThreadContext *tc, MVMString *name)
             "<PRIVATE-USE-",
             "TANGUT IDEOGRAPH-"
         };
+        size_t cname_len = strlen((const char *) cname );
         int i;
         for (i = 0; i < prefixes_len; i++) {
-            int str_len = strlen(prefixes[i]);
+            size_t str_len = strlen(prefixes[i]);
             if (cname_len <= str_len)
                 continue;
             /* Make sure to catch conditions which strtoll is ok with but we
@@ -712,7 +709,7 @@ MVMGrapheme32 MVM_unicode_lookup_by_name(MVMThreadContext *tc, MVMString *name)
             if (!strncmp(cname, prefixes[i], str_len)) {
                 char *reject = NULL;
                 MVMint64 rtrn = strtol(cname + strlen(prefixes[i]), &reject, 16);
-                if (prefixes[i][0] == '<' && *reject == '>' && reject - cname + 1 == cname_len) {
+                if (prefixes[i][0] == '<' && *reject == '>' && (size_t)(reject - cname + 1) == cname_len) {
                     MVM_free(cname);
                     return rtrn;
                 }
@@ -724,7 +721,7 @@ MVMGrapheme32 MVM_unicode_lookup_by_name(MVMThreadContext *tc, MVMString *name)
         }
     }
     MVM_free(cname);
-    return result ? result->codepoint : -1;
+    return result ? result->value : -1;
 }
 /* Quickly determines the length of a number 6.5x faster than doing log10 after
  * compiler optimization */
@@ -751,16 +748,9 @@ MVMString * MVM_unicode_get_name(MVMThreadContext *tc, MVMint64 codepoint) {
         name_len = strlen(name);
     /* Look up name. */
     else {
-        MVMuint32 codepoint_row = MVM_codepoint_to_row_index(tc, codepoint);
+        MVMint32 codepoint_row = MVM_codepoint_to_row_index(tc, codepoint);
         if (codepoint_row != -1) {
             name = codepoint_names[codepoint_row];
-            if (!name) {
-                while (codepoint_row && !codepoint_names[codepoint_row])
-                    codepoint_row--;
-                name = codepoint_names[codepoint_row];
-                if (name && name[0] != '<')
-                    name = NULL;
-            }
         }
         if (!name) {
             /* U+FDD0..U+FDEF and the last two codepoints of each block
@@ -891,86 +881,78 @@ MVMuint32 MVM_unicode_get_case_change(MVMThreadContext *tc, MVMCodepoint codepoi
     return 0;
 }
 
-/* XXX make all the statics members of the global MVM instance instead? */
-static MVMUnicodeNameRegistry *property_codes_by_names_aliases;
-static MVMUnicodeGraphemeNameRegistry *property_codes_by_seq_names;
-
 static void generate_property_codes_by_names_aliases(MVMThreadContext *tc) {
     MVMuint32 num_names = num_unicode_property_keypairs;
 
-    while (num_names--) {
-        MVMUnicodeNameRegistry *entry = MVM_malloc(sizeof(MVMUnicodeNameRegistry));
-        entry->name = (char *)unicode_property_keypairs[num_names].name;
-        entry->codepoint = unicode_property_keypairs[num_names].value;
-        HASH_ADD_KEYPTR(hash_handle, property_codes_by_names_aliases,
-            entry->name, strlen(entry->name), entry);
+    uv_mutex_lock(&tc->instance->mutex_property_codes_hash_setup);
+    if (MVM_uni_hash_is_empty(tc, &tc->instance->property_codes_by_names_aliases)) {
+        MVM_uni_hash_build(tc, &tc->instance->property_codes_by_names_aliases, num_names);
+
+        while (num_names--) {
+            MVM_uni_hash_insert(tc, &tc->instance->property_codes_by_names_aliases,
+                                unicode_property_keypairs[num_names].name,
+                                unicode_property_keypairs[num_names].value);
+        }
     }
+    uv_mutex_unlock(&tc->instance->mutex_property_codes_hash_setup);
 }
 static void generate_property_codes_by_seq_names(MVMThreadContext *tc) {
     MVMuint32 num_names = num_unicode_seq_keypairs;
 
-    while (num_names--) {
-        MVMUnicodeGraphemeNameRegistry *entry = MVM_malloc(sizeof(MVMUnicodeGraphemeNameRegistry));
-        entry->name = (char *)uni_seq_pairs[num_names].name;
-        entry->structindex = uni_seq_pairs[num_names].value;
-        HASH_ADD_KEYPTR(hash_handle, property_codes_by_seq_names,
-            entry->name, strlen(entry->name), entry);
+    uv_mutex_lock(&tc->instance->mutex_property_codes_hash_setup);
+    if (MVM_uni_hash_is_empty(tc, &tc->instance->property_codes_by_seq_names)) {
+        MVM_uni_hash_build(tc, &tc->instance->property_codes_by_seq_names, num_names);
+
+        while (num_names--) {
+            MVM_uni_hash_insert(tc, &tc->instance->property_codes_by_seq_names,
+                                uni_seq_pairs[num_names].name,
+                                uni_seq_pairs[num_names].value);
+        }
     }
+    uv_mutex_unlock(&tc->instance->mutex_property_codes_hash_setup);
 }
 
 MVMint64 MVM_unicode_name_to_property_code(MVMThreadContext *tc, MVMString *name) {
     MVMuint64 size;
     char *cname = MVM_string_ascii_encode(tc, name, &size, 0);
-    MVMUnicodeNameRegistry *result;
-    if (!property_codes_by_names_aliases) {
+    if (MVM_uni_hash_is_empty(tc, &tc->instance->property_codes_by_names_aliases)) {
         generate_property_codes_by_names_aliases(tc);
     }
-    HASH_FIND(hash_handle, property_codes_by_names_aliases, cname, strlen((const char *)cname), result);
-    MVM_free(cname); /* not really codepoint, really just an index */
-    return result ? result->codepoint : 0;
+    struct MVMUniHashEntry *result = MVM_uni_hash_fetch(tc, &tc->instance->property_codes_by_names_aliases, cname);
+    MVM_free(cname);
+    return result ? result->value : 0;
 }
 
-static void generate_unicode_property_values_hashes(MVMThreadContext *tc) {
-    MVMUnicodeNameRegistry **hash_array = MVM_calloc(MVM_NUM_PROPERTY_CODES, sizeof(MVMUnicodeNameRegistry *));
+void MVM_unicode_init(MVMThreadContext *tc) {
+    /* A bit of a hack - we're relying on the implementation detail that zeroes
+     * is a valid start state for the hash. */
+    MVMUniHashTable *hash_array = MVM_calloc(MVM_NUM_PROPERTY_CODES, sizeof(MVMUniHashTable));
     MVMuint32 index = 0;
-    MVMUnicodeNameRegistry *entry = NULL, *binaries = NULL;
     for ( ; index < num_unicode_property_value_keypairs; index++) {
         MVMint32 property_code = unicode_property_value_keypairs[index].value >> 24;
-        entry = MVM_malloc(sizeof(MVMUnicodeNameRegistry));
-        entry->name = (char *)unicode_property_value_keypairs[index].name;
-        entry->codepoint = unicode_property_value_keypairs[index].value & 0xFFFFFF;
-        HASH_ADD_KEYPTR(hash_handle, hash_array[property_code],
-            entry->name, strlen(entry->name), entry);
+        MVM_uni_hash_insert(tc, &hash_array[property_code],
+                            unicode_property_value_keypairs[index].name,
+                            unicode_property_value_keypairs[index].value & 0xFFFFFF);
     }
     for (index = 0; index < MVM_NUM_PROPERTY_CODES; index++) {
-        if (!hash_array[index]) {
-            if (!binaries) {
-                MVMUnicodeNamedValue yes[8] = { {"T",1}, {"Y",1},
-                    {"Yes",1}, {"yes",1}, {"True",1}, {"true",1}, {"t",1}, {"y",1} };
-                MVMUnicodeNamedValue no [8] = { {"F",0}, {"N",0},
-                    {"No",0}, {"no",0}, {"False",0}, {"false",0}, {"f",0}, {"n",0} };
-                MVMuint8 i;
-                for (i = 0; i < 8; i++) {
-                    entry = MVM_malloc(sizeof(MVMUnicodeNameRegistry));
-                    entry->name = (char *)yes[i].name;
-                    entry->codepoint = yes[i].value;
-                    HASH_ADD_KEYPTR(hash_handle, binaries, yes[i].name, strlen(yes[i].name), entry);
-                }
-                for (i = 0; i < 8; i++) {
-                    entry = MVM_malloc(sizeof(MVMUnicodeNameRegistry));
-                    entry->name = (char *)no[i].name;
-                    entry->codepoint = no[i].value;
-                    HASH_ADD_KEYPTR(hash_handle, binaries, no[i].name, strlen(no[i].name), entry);
-                }
+        if (MVM_uni_hash_is_empty(tc, &hash_array[index])) {
+            MVMUnicodeNamedValue yes[8] = { {"T",1}, {"Y",1},
+                {"Yes",1}, {"yes",1}, {"True",1}, {"true",1}, {"t",1}, {"y",1} };
+            MVMUnicodeNamedValue no [8] = { {"F",0}, {"N",0},
+                {"No",0}, {"no",0}, {"False",0}, {"false",0}, {"f",0}, {"n",0} };
+            MVMuint8 i;
+            for (i = 0; i < 8; i++) {
+                MVM_uni_hash_insert(tc, &hash_array[index], yes[i].name, yes[i].value);
+            }
+            for (i = 0; i < 8; i++) {
+                MVM_uni_hash_insert(tc, &hash_array[index], no[i].name, no[i].value);
             }
-            hash_array[index] = binaries;
         }
     }
-    unicode_property_values_hashes = hash_array;
+    tc->instance->unicode_property_values_hashes = hash_array;
 }
-MVMint32 unicode_cname_to_property_value_code(MVMThreadContext *tc, MVMint64 property_code, const char *cname, MVMuint64 cname_length) {
+static MVMint32 unicode_cname_to_property_value_code(MVMThreadContext *tc, MVMint64 property_code, const char *cname, MVMuint64 cname_length) {
     char *out_str = NULL;
-    MVMUnicodeNameRegistry *result = NULL;
                                    /* number + dash + property_value + NULL */
     MVMuint64 out_str_length = length_of_num(property_code) + 1 + cname_length + 1;
     if (1024 < out_str_length)
@@ -979,8 +961,10 @@ MVMint32 unicode_cname_to_property_value_code(MVMThreadContext *tc, MVMint64 pro
     out_str = alloca(sizeof(char) * out_str_length);
     snprintf(out_str, out_str_length, "%"PRIi64"-%s", property_code, cname);
 
-    HASH_FIND(hash_handle, unicode_property_values_hashes[property_code], out_str, out_str_length - 1, result);
-    return result ? result->codepoint : 0;
+    struct MVMUniHashEntry *result = MVM_uni_hash_fetch(tc,
+                                                        &tc->instance->unicode_property_values_hashes[property_code],
+                                                        out_str);
+    return result ? result->value : 0;
 }
 MVMint64 MVM_unicode_name_to_property_value_code(MVMThreadContext *tc, MVMint64 property_code, MVMString *name) {
     if (property_code <= 0 || MVM_NUM_PROPERTY_CODES <= property_code)
@@ -1017,65 +1001,12 @@ MVMCodepoint MVM_unicode_find_primary_composite(MVMThreadContext *tc, MVMCodepoi
     return 0;
 }
 
-static uv_mutex_t property_hash_count_mutex;
-static int property_hash_count = 0;
-static uv_once_t property_hash_count_guard = UV_ONCE_INIT;
-
-static void setup_property_mutex(void)
-{
-    uv_mutex_init(&property_hash_count_mutex);
-}
-
-void MVM_unicode_init(MVMThreadContext *tc)
-{
-    uv_once(&property_hash_count_guard, setup_property_mutex);
-
-    uv_mutex_lock(&property_hash_count_mutex);
-    if (property_hash_count == 0) {
-        generate_unicode_property_values_hashes(tc);
-    }
-    property_hash_count++;
-    uv_mutex_unlock(&property_hash_count_mutex);
-}
-
-void MVM_unicode_release(MVMThreadContext *tc)
-{
-    uv_mutex_lock(&property_hash_count_mutex);
-    property_hash_count--;
-    if (property_hash_count == 0) {
-        int i;
-
-        for (i = 0; i < MVM_NUM_PROPERTY_CODES; i++) {
-            MVMUnicodeNameRegistry *entry = NULL;
-            int j;
-
-            if (!unicode_property_values_hashes[i]) {
-                continue;
-            }
-
-            for(j = i + 1; j < MVM_NUM_PROPERTY_CODES; j++) {
-                if (unicode_property_values_hashes[i] == unicode_property_values_hashes[j]) {
-                    unicode_property_values_hashes[j] = NULL;
-                }
-            }
-            MVM_HASH_DESTROY(tc, hash_handle, MVMUnicodeNameRegistry, unicode_property_values_hashes[i]);
-            assert(!unicode_property_values_hashes[i]);
-        }
-
-        MVM_free(unicode_property_values_hashes);
-
-        unicode_property_values_hashes = NULL;
-    }
-    uv_mutex_unlock(&property_hash_count_mutex);
-}
 /* Looks up a codepoint sequence or codepoint by name (case insensitive).
  First tries to look it up by codepoint with MVM_unicode_lookup_by_name and if
  not found as a named codepoint, lazily constructs a hash of the codepoint
  sequences and looks up the sequence name */
 MVMString * MVM_unicode_string_from_name(MVMThreadContext *tc, MVMString *name) {
     MVMString * name_uc = MVM_string_uc(tc, name);
-    char * cname = NULL;
-    MVMUnicodeGraphemeNameRegistry *result;
 
     MVMGrapheme32 result_graph = MVM_unicode_lookup_by_name(tc, name_uc);
     /* If it's just a codepoint, return that */
@@ -1085,17 +1016,19 @@ MVMString * MVM_unicode_string_from_name(MVMThreadContext *tc, MVMString *name)
     /* Otherwise look up the sequence */
     else {
         const MVMint32 *uni_seq = NULL;
-        cname = MVM_string_utf8_encode_C_string(tc, name_uc);
-        if (!property_codes_by_seq_names) {
+        char *cname = MVM_string_utf8_encode_C_string(tc, name_uc);
+        if (MVM_uni_hash_is_empty(tc, &tc->instance->property_codes_by_seq_names)) {
             generate_property_codes_by_seq_names(tc);
         }
-        HASH_FIND(hash_handle, property_codes_by_seq_names, cname, strlen((const char *)cname), result);
+        struct MVMUniHashEntry *result = MVM_uni_hash_fetch(tc,
+                                                            &tc->instance->property_codes_by_seq_names,
+                                                            cname);
         MVM_free(cname);
         /* If we can't find a result return an empty string */
         if (!result)
             return tc->instance->str_consts.empty;
 
-        uni_seq = uni_seq_enum[result->structindex];
+        uni_seq = uni_seq_enum[result->value];
         /* The first element is the number of codepoints in the sequence */
         return MVM_unicode_codepoints_c_array_to_nfg_string(tc, (MVMCodepoint *) uni_seq + 1, uni_seq[0]);
     }
